/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_test_hs_mc_driver.c
 **
 **  Tests the H-S M&C Driver
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hs_mc_driver.h"
#include "gcspy_main_server.h"
#include "gcspy_timer.h"

#ifdef _SPARC_
#include <thread.h>
#endif //_SPARC_

#ifdef _LINUX_
#include <pthread.h>
#endif //_LINUX_

#define MAX_LEN      ( 128 * 1024 )
#define BLOCK_SIZE    ( 32 * 1024 )
#define CARD_SIZE           ( 512 )

static gcspy_main_server_t server;
static mc_driver_t mcDriver;

#define ZERO      0
#define ONE     ( 2 * 1024 * 1024 )
#define TWO     ( 4 * 1024 * 1024 )

#define START_GC_EVENT      0
#define END_GC_EVENT        1

#ifdef  __cplusplus
extern "C" {
#endif //__cplusplus
static void *
mainLoop (void *arg) {
  int count = 0;
  char *limit;
  char *end;
  int event;

  while ( 1 ) {
    gcspy_wait(1000);

    if ((count % 2) == 0) {
      event = START_GC_EVENT;
    } else {
      event = END_GC_EVENT;
    }

    if (gcspy_mainServerIsConnected(&server, event)) {
      gcspy_mainServerStartCompensationTimer(&server);

      switch (count % 4) {
      case 0:
	limit = (char *) ZERO;
	end = (char *) ONE;
	break;
      case 1:
	limit = (char *) (ZERO + 520 * 1024);
	end = (char *) ONE;
	break;
      case 2:
	limit = (char *) (ONE + 30 * 1024);
	end = (char *) TWO;
	break;
      case 3:
	limit = (char *) (TWO - 100 * 1024);
	end = (char *) TWO;
	break;
      }

      hsMCDriverZero(&mcDriver, end);
      hsMCDriverSetLimit(&mcDriver, limit);

      gcspy_mainServerStopCompensationTimer(&server);

      if (gcspy_mainServerIsConnected(&server, event)) {
	printf("CONNECTED\n");
	hsMCDriverSend(&mcDriver, event);
      }
    }
    gcspy_mainServerSafepoint(&server, event);
    ++count;
  }
}
#ifdef  __cplusplus
}
#endif //__cplusplus

int
main (int argc, char *argv[]) {
  char generalInfo[256];
  gcspy_date_time_t time;

#ifdef _SPARC_
  thread_t tid;
#endif //_SPARC_

#ifdef _LINUX_
  pthread_t tid;
#endif //_LINUX_

  gcspy_gc_driver_t *driver;
  int port;
  int res;

  if (argc != 2) {
    gcspy_raise_error("Wrong number of arguments");
  }
  port = atoi(argv[1]);

  printf("-- Initialising main server on port %d\n", port);
  gcspy_mainServerInit(&server, port, MAX_LEN, "M&C Driver Test", 1);

  time = gcspy_getDateTime();
  strcpy(generalInfo, "GCspy Test\n\n");
  strcat(generalInfo, "Start Time:\n  ");
  strcpy(generalInfo, gcspy_formatDateTime(time));
  strcat(generalInfo, "\nApplication:\n  M&C Driver Test\n\n");
  strcat(generalInfo, "1 Space");
  gcspy_mainServerSetGeneralInfo(&server, generalInfo);

  printf("--   Setting event 0\n");
  gcspy_mainServerAddEvent(&server, START_GC_EVENT, "Start M&C GC");
  printf("--   Setting event 1\n");
  gcspy_mainServerAddEvent(&server, END_GC_EVENT, "End M&C GC");

  printf("--   Setting up driver 0\n");
  driver = gcspy_mainServerAddDriver(&server);

  hsMCDriverInit(&mcDriver, driver, 
		 "Single Generation",
		 BLOCK_SIZE,
		 CARD_SIZE,
		 (char *) ZERO,
		 (char *) ONE);

#ifdef _SPARC_
  res = thr_create(NULL, 0, mainLoop, NULL, THR_BOUND, &tid);
#endif //_SPARC_

#ifdef _LINUX_
  res = pthread_create(&tid, NULL, mainLoop, NULL);
#endif //_LINUX_

  if (res != 0) {
    printf("Couldn't create thread.\n");
    exit(-1);
  }

  gcspy_mainServerMainLoop(&server);
}
