/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_color_db.hpp
 **
 **  Maps colors to/from names
 **/

#ifndef _GCSPY_COLOR_DB_HPP_

#define _GCSPY_COLOR_DB_HPP_

#include "gcspy_types.hpp"
#include "gcspy_utils.hpp"

class gcspyColor {

private:
  unsigned char _red;
  unsigned char _green;
  unsigned char _blue;

public:
  gcspyColor (unsigned char red, 
	      unsigned char green,
	      unsigned char blue);

  inline gcspyColor () { };

  inline unsigned char get_red()    { return _red; }
  inline unsigned char get_green()  { return _green; }
  inline unsigned char get_blue()   { return _blue; }

  inline void set_red (unsigned char red)      { _red   = red; }
  inline void set_green (unsigned char green)  { _green = green; }
  inline void set_blue (unsigned char blue)    { _blue  = blue; }

  inline gcspy_bool_t is_equal (gcspyColor *color) {
    return (_red == color->_red) &&
      (_green == color->_green) &&
      (_blue == color->_blue);
  }

};


class gcspyColorDB {

private:
  enum PrivateConstants {
    default_index = 0
  };

public:
  inline static int get_default_index() { return default_index; }
  inline static gcspyColor *get_default_color() {
    return get_color(default_index);
  }

  static gcspyColor *get_color (int index);
  static gcspyColor *get_color (const char *name);
  static const char *get_name (int index);
  static const char *get_name (gcspyColor *color);
  static int get_index (gcspyColor *color);
  static gcspy_length_t get_length (void);
  static void dump (void);

};

#endif //_GCSPY_COLOR_DB_HPP_
