/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_color_db.cpp
 **
 **  Maps colors to/from names
 **/

#include <stdio.h>
#include <string.h>

#include "gcspy_color_db.hpp"

enum Constants {
  length = 16
};

static gcspyColor colors[] = {
  gcspyColor(   0,   0,   0 ),
  gcspyColor(   0,   0, 255 ),
  gcspyColor(   0, 255, 255 ),
  gcspyColor(  64,  64,  64 ),
  gcspyColor( 128, 128, 128 ),
  gcspyColor(   0, 255,   0 ),
  gcspyColor( 192, 192, 192 ),
  gcspyColor( 255,   0, 255 ),
  gcspyColor( 160, 160, 160 ),
  gcspyColor(   0,   0, 150 ),
  gcspyColor( 230, 230, 230 ),
  gcspyColor( 255, 200,   0 ),
  gcspyColor( 255, 175, 175 ),
  gcspyColor( 255,   0,   0 ),
  gcspyColor( 255, 255, 255 ),
  gcspyColor( 255, 255,   0 )
};

static const char *names[] = {
	"Black",
	"Blue",
	"Cyan",
	"Dark Gray",
	"Gray",
	"Green",
	"Light Gray",
	"Magenta",
	"Mid Gray",
	"Navy Blue",
	"Off White",
	"Orange",
	"Pink",
	"Red",
	"White",
	"Yellow"
};

gcspyColor::gcspyColor (unsigned char red,
			unsigned char green,
			unsigned char blue) {
  set_red(red);
  set_green(green);
  set_blue(blue);
}

gcspyColor *
gcspyColorDB::get_color (int index) {
  gcspy_assert((index >= 0) && (index < length));
  return &colors[index];
}

gcspyColor *
gcspyColorDB::get_color (const char *name) {
  if (name == NULL)
    return get_default_color();
  for (int i = 0; i < length; ++i) {
    if (!strcasecmp(name, names[i])) {
      return &colors[i];
    }
  }
  return get_default_color();
}

const char *
gcspyColorDB::get_name (int index) {
  gcspy_assert((index >= 0) && (index < length));
  return names[index];
}

const char *
gcspyColorDB::get_name (gcspyColor *color) {
  return names[get_index(color)];
}

int
gcspyColorDB::get_index (gcspyColor *color) {
  if (color == NULL)
    return -1;
  for (int i = 0; i < length; ++i) {
    if (colors[i].is_equal(color))
      return i;
  }
  return -1;
}

gcspy_length_t
gcspyColorDB::get_length () {
  return length;
}

void
gcspyColorDB::dump () {
  for (int i = 0; i < length; ++i) {
    gcspyColor *color = gcspyColorDB::get_color(i);
    printf("%d.%s: %u,%u,%u\n",
	   i, gcspyColorDB::get_name(i),
	   color->get_red(),
	   color->get_green(),
	   color->get_blue());
  }
}
