/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  hs_old_driver.cpp
 **
 **  Driver for the HS old generation M&C collector
 **/

#include "hs_old_driver.hpp"

void
hsOldDriver::retrieve_data_arrays () {
  hsMCDriver::retrieve_data_arrays();
  _direct_alloc_data    = _direct_alloc_stream->get_int_data();
  _promotion_data       = _promotion_stream->get_int_data();
  _refs_to_left_data    = _refs_to_left_stream->get_int_data();
  _refs_to_right_data   = _refs_to_right_stream->get_int_data();
  _refs_to_perm_data    = _refs_to_perm_stream->get_int_data();
}

void
hsOldDriver::init_summaries (gcspy_size_t area_size) {
  hsMCDriver::init_summaries(area_size);
  _direct_alloc_summary[1] = area_size;
  _promotion_summary[1]    = area_size;
}

hsOldDriver::hsOldDriver (gcspy_size_t block_size,
			  gcspy_size_t card_size,
			  char *start,
			  char *end)
  : hsMCDriver("Old Generation", block_size, card_size, start, end) {

  add_used_space_stream();
  add_dead_space_stream();
  add_card_table_stream();

  _direct_alloc_stream = new gcspyStream(gcspyStream::int_type,
				      "Direct Alloc",
				      0, block_size, 0, 0,
				     "Direct Alloc: ", " bytes",
				     gcspyStream::presentation_percent,
				     gcspyStream::paint_style_zero,
				     0, gcspyColorDB::get_color("Magenta"), 1);
  add_stream(_direct_alloc_stream);
  _direct_alloc_summary = _direct_alloc_stream->get_summary();

  _promotion_stream = new gcspyStream(gcspyStream::int_type,
				     "Promotion",
				     0, block_size, 0, 0,
				     "Promotion: ", " bytes",
				     gcspyStream::presentation_percent,
				     gcspyStream::paint_style_zero,
				     0, gcspyColorDB::get_color("Magenta"), 1);
  add_stream(_promotion_stream);
  _promotion_summary = _promotion_stream->get_summary();

  add_object_stream();
  add_marking_stream();
  add_refs_stream();

  _refs_to_left_stream = new gcspyStream(gcspyStream::int_type,
				     "Reference Fields (Left)",
				     0,
				     gcspyContArea::refs_per_block(block_size),
				     0, 0,
				     "Reference Fields: ", " (Left)",
				     gcspyStream::presentation_plus,
				     gcspyStream::paint_style_zero,
				     0, gcspyColorDB::get_color("Green"), 1);
  add_stream(_refs_to_left_stream);
  _refs_to_left_summary = _refs_to_left_stream->get_summary();

  _refs_to_right_stream = new gcspyStream(gcspyStream::int_type,
				     "Reference Fields (Right)",
				     0,
				     gcspyContArea::refs_per_block(block_size),
				     0, 0,
				     "Reference Fields: ", " (Right)",
				     gcspyStream::presentation_plus,
				     gcspyStream::paint_style_zero,
				     0, gcspyColorDB::get_color("Green"), 1);
  add_stream(_refs_to_right_stream);
  _refs_to_right_summary = _refs_to_right_stream->get_summary();

  add_refs_to_young_stream();

  _refs_to_perm_stream = new gcspyStream(gcspyStream::int_type,
				     "Reference Fields (Perm)",
				     0,
				     gcspyContArea::refs_per_block(block_size),
				     0, 0,
				     "Reference Fields: ", " (Perm)",
				     gcspyStream::presentation_plus,
				     gcspyStream::paint_style_zero,
				     0, gcspyColorDB::get_color("Green"), 1);
  add_stream(_refs_to_perm_stream);
  _refs_to_perm_summary = _refs_to_perm_stream->get_summary();

  add_internal_refs_stream();

  retrieve_data_arrays();
}
